#!/usr/local/bin/php -f
<?php
/**
 * $Horde: rakim/scripts/chatMonitor.php,v 1.13 2004/12/28 07:06:27 chuck Exp $
 *
 * This is a command-line script to log guests and support techs out
 * of the system. You should place a line similar to the following in
 * your crontab:
 *
 *     * * * * * /path/to/rakim/scripts/chatMonitor.php
 *
 * This will run it every minute, making sure that dropped calls don't
 * go very long without being caught.
 *
 * On a Windows platform, you should be able to use the Task Scheduler
 * to have the script run every minute in the same fashion.
 */

@define('AUTH_HANDLER', true);
@define('HORDE_BASE', dirname(__FILE__) . '/../..');
@define('RAKIM_BASE', dirname(__FILE__) . '/..');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

// Now go ahead and load Rakim.
require_once RAKIM_BASE . '/lib/base.php';

// Consider stale anyone that hasn't refreshed in 60 seconds. Note
// that this doesn't mean they've sent a message in 60 seconds; their
// browser should be updating every 5 seconds regardless of message
// traffic.
$staleTime = time() - 60;

monitorGuests();
monitorTechs();


/**
 * Check for any guests that have closed their browser or had
 * technical problems.
 */
function monitorGuests()
{
    global $db, $staleTime;

    $findStaleGuestsQuery = 'SELECT guest_name, guest_id, guest_refresh_time, guest_login_time FROM rakim_guests 
        WHERE guest_refresh_time < ' . (int)$staleTime;

    $staleGuests = $db->query($findStaleGuestsQuery);
    if (is_a($staleGuests, 'PEAR_Error')) {
        Horde::logMessage($staleGuests, __FILE__, __LINE__, PEAR_LOG_ERR);
        return false;
    }

    while ($row = $staleGuests->fetchRow(DB_FETCHMODE_ASSOC)) {
        sendGuestMessage(_("This guest appears to have either left the system or experienced technical difficulties."), $row['guest_id'], 'f', 'f', 'rakim_admin', _("Administrator")); 

        $exitGuestQuery = "UPDATE rakim_queues SET queue_status = 'exit' WHERE guest_id = " . (int)$row['guest_id'];
        $result = $db->query($exitGuestQuery);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }

        $guestLogoutQuery = 'UPDATE rakim_guests SET guest_logoff_time = ' . time() . ', guest_refresh_time = NULL WHERE guest_id = ' . (int)$row['guest_id'];
        $result = $db->query($guestLogoutQuery);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        }
    }
}

/**
 * Check for techs that have logged out, left, or had technical
 * difficulties.
 */
function monitorTechs()
{
    global $db, $rakim, $registry, $conf, $staleTime;

    $findStaleTechsQuery = 'SELECT user_uid, user_last_active FROM rakim_support_personnel
        WHERE user_last_active < ' . (int)$staleTime;

    $staleTechs = $db->query($findStaleTechsQuery);
    if (is_a($staleTechs, 'PEAR_Error')) {
        Horde::logMessage($staleTechs, __FILE__, __LINE__, PEAR_LOG_ERR);
        return false;
    } elseif (!$staleTechs->numRows()) {
        return true;
    }

    // See if anyone else is available.
    $activeTechs = $rakim->getActivePersonnel();

    while ($row = $staleTechs->fetchRow(DB_FETCHMODE_ASSOC)) {
        // Say adios.
        $userprefs = &Prefs::singleton($conf['prefs']['driver'],
                                       $registry->getApp(),
                                       $row['user_uid'], '', null, false);
        $userprefs->retrieve();
        $farewell = $userprefs->getValue('support_farewell');
        if ($farewell) {
            sendTechMessage($farewell, 0, $row['user_uid']);
        }

        $rakim->updateLastActive(true, $row['user_uid']);

        $findStaleQueuesQuery = 'SELECT * FROM rakim_queues WHERE user_uid = ' . $db->quote($row['user_uid']) . ' AND queue_status != \'exit\'';
        $staleQueues = $db->query($findStaleQueuesQuery);
        if (is_a($staleQueues, 'PEAR_Error')) {
            Horde::logMessage($staleQueues, __FILE__, __LINE__, PEAR_LOG_ERR);
            continue;
        }

        if (!$staleQueues->numRows()) {
            continue;
        }

        // Get the user's Identity for sending messages.
        require_once 'Horde/Identity.php';
        $identity = &Identity::singleton('none', $row['user_uid']);
        $fullname = $identity->getValue('fullname') ? $identity->getValue('fullname') : $row['user_uid'];

        while ($row2 = $staleQueues->fetchRow(DB_FETCHMODE_ASSOC)) {
            $findOtherQueuesQuery = 'SELECT count(*) FROM rakim_queues WHERE guest_id = ' . (int)$row2['guest_id'] . ' AND user_uid != ' . $db->quote($row['user_uid']) . ' AND queue_status != \'exit\'';
            $otherQueues = $db->getOne($findOtherQueuesQuery);

            if ($otherQueues) {
                $updateQueueQuery = 'UPDATE rakim_queues
                    SET queue_status = \'exit\'
                    WHERE guest_id = ' . (int)$row2['guest_id'] . '
                          AND user_uid = ' . $db->quote($row['user_uid']);

                $result = $db->query($updateQueueQuery);
                if (is_a($result, 'PEAR_Error')) {
                    Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                }

                sendGuestMessage(sprintf(_("%s has experienced technical difficulties and dropped this session. Apologies for any inconvenience."), $fullname), $row2['guest_id'], 'f', 'f', 'rakim_admin', _("Administrator"));
            } elseif (count($activeTechs)) {
                referGuest($row2['guest_id'], $row['user_uid'], $fullname, true);
            } else {
                sendGuestMessage(sprintf(_("%s has experienced technical difficulties and dropped this session. There are not any other personnel available at this time; please try again in a few minutes or submit your question via email, using the link at the top of your screen. Apologies for any inconvenience."), $fullname), $row2['guest_id'], 'f', 't', 'rakim_admin', _("Administrator"));
            }
        }
    }
}

/**
 * Refer a guest to anyone else available.
 *
 * @param integer $guest_id  The guest to refer.
 * @param string  $user_uid  The user who was responsible for the guest.
 * @param string  $fullname  The name of the user who left the session.
 * @param boolean $clear     Close out any other queues for this guest?
 */
function referGuest($guest_id = 0, $user_uid, $fullname, $clear = true)
{
    global $db;

    // First, make sure any other queues for this guest (probably
    // stale) are closed to 'exit';
    $updateQueueQuery = 'UPDATE rakim_queues
        SET queue_status = \'exit\'
        WHERE guest_id = ' . (int)$guest_id;

    $result = $db->query($updateQueueQuery);
    if (is_a($result, 'PEAR_Error')) {
        Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
    }

    $queue_id = $db->nextId('rakim_queue_id');
    if (is_a($queue_id, 'PEAR_Error')) {
        return $queue_id;
    }

    $referralQuery = 'INSERT INTO rakim_queues (queue_id, guest_id, user_uid, queue_status, referrer_user_uid)
        VALUES (' . (int)$queue_id . ', ' . (int)$guest_id . ', NULL' .
        ", 'referral', " . $db->quote($user_uid) . ')';

    $referral_timeQuery = "UPDATE rakim_guests 
        SET guest_referral_time = " . time() . "
        WHERE guest_id = " . (int)$guest_id;

    Horde::logMessage($referralQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);
    Horde::logMessage($referral_timeQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

    $result_i = $db->query($referralQuery);
    $result_u = $db->query($referral_timeQuery);
    if (is_a($result_i, 'PEAR_Error')) {
        Horde::logMessage($result_i, __FILE__, __LINE__, PEAR_LOG_ERR);
    }
    if (is_a($result_u, 'PEAR_Error')) {
        Horde::logMessage($result_u, __FILE__, __LINE__, PEAR_LOG_ERR);
    }

    sendGuestMessage(sprintf(_("%s has experienced technical difficulties and dropped this session. Please wait while you are referred to another support tech."), $fullname), $guest_id, 'f', 'f', 'rakim_admin', _("Administrator"));

    $updateQueueQuery = "UPDATE rakim_queues SET queue_status = 'exit'
        WHERE guest_id = " . (int)$guest_id . ' AND user_uid = ' . $db->quote($user_uid);

    $result = $db->query($updateQueueQuery);
    if (is_a($result, 'PEAR_Error')) {
        Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
    }
}
