#!/usr/local/bin/php -f
<?php
/**
 * $Horde: rakim/scripts/emailMonitor.php,v 1.10 2004/10/19 00:46:04 chuck Exp $
 *
 * This is a command-line script to check for emails that have gone
 * too long without being answered or closed.
 *
 *     0 2 * * * /path/to/rakim/scripts/emailMonitor.php
 *
 * This will run it every morning at 2am.
 *
 * On a Windows platform, you should be able to use the Task Scheduler
 * to have the script run every night in the same fashion.
 */

@define('AUTH_HANDLER', true);
@define('HORDE_BASE', dirname(__FILE__) . '/../..');
@define('RAKIM_BASE', dirname(__FILE__) . '/..');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

// Now go ahead and load Rakim.
require_once RAKIM_BASE . '/lib/base.php';

// Consider stale emails older than this timestamp:
$staleTime = time() - ($conf['stats']['warn_email'] * 24 * 60 * 60);

staleQuestions();


/**
 *
 */
function staleQuestions()
{
    global $db, $staleTime;

    $findStaleQuestionsQuery = 'SELECT * FROM rakim_emails
        WHERE (email_status = \'open\' AND email_time_asked < ' . $staleTime . ')
           OR (email_status != \'closed\' AND email_status != \'open\' AND email_time_claimed < ' . $staleTime . ')';

    $staleQuestions = $db->query($findStaleQuestionsQuery);
    if (is_a($staleQuestions, 'PEAR_Error')) {
        Horde::logMessage($staleQuestions, __FILE__, __LINE__, PEAR_LOG_ERR);
        return false;
    }

    $open = array();
    $claimed = array();
    while ($row = $staleQuestions->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ($row['email_status'] == 'open') {
            $open[] = array('id' => $row['email_id'],
                            'from' => $row['guest_name'],
                            'text' => substr($row['email_text'], 0, 200),
                            'time' => $row['email_time_asked']);
        } else {
            $claimed[] = array('id' => $row['email_id'],
                               'from' => $row['guest_name'],
                               'text' => substr($row['email_text'], 0, 200),
                               'time' => $row['email_time_claimed'],
                               'status' => $row['email_status']);
        }
    }

    if (count($open) || count($claimed)) {
        generateReport($open, $claimed);
    }
}

/**
 * Generate an email report.
 *
 * @param array $open     Requests that have been open too long.
 * @param array $claimed  Requests that have been claimed without closing too long.
 */
function generateReport($open = array(), $claimed = array())
{
    global $conf, $registry;

    require_once 'Horde/MIME/Headers.php';
    require_once 'Horde/MIME/Message.php';
    require_once 'Horde/MIME/Part.php';

    // Email header.
    $text = sprintf(_("This is a generated report of email requests that have gone too long without action. It was sent to %s, as configured by the %s administrator. If you do not think you should be receiving this email, please contact that administrator."),
                    $conf['stats']['warn_email_admin'], $registry->get('name'));
    $text = String::wrap($text, 76);

    if (count($open)) {
        $text .= "\n\n---------------------------------------------------------------------------\n\n";
        $text .= String::wrap(sprintf(_("There are %s requests that have been open for more than %s days without being claimed:"),
                                    count($open), $conf['stats']['warn_email']), 76);
        $text .= "\n\n";

        $i = 0;
        foreach ($open as $request) {
            $text .= sprintf(_("From:    %s"), $request['from']) . "\n";
            $text .= sprintf(_("Asked:   %s"), strftime('%x %X', $request['time'])) . "\n";
            $text .= sprintf(_("URL:     %s"), Horde::applicationUrl('emailView.php?id=' . $request['id'], true, -1)) . "\n";
            $text .= _("First few lines:") . "\n" . String::wrap($request['text'], 76) . "\n";

            $i++;
            if ($i != 0 && $i != count($open)) {
                $text .= "\n--------------\n\n";
            }
        }
    }

    if (count($claimed)) {
        $text .= "\n\n---------------------------------------------------------------------------\n\n";
        $text .= String::wrap(sprintf(_("There are %s requests that have been claimed for more than %s days without being closed:"),
                                    count($claimed), $conf['stats']['warn_email']), 76);
        $text .= "\n\n";

        $i = 0;
        foreach ($claimed as $request) {
            $text .= sprintf(_("From:    %s"), $request['from']) . "\n";
            $text .= sprintf(_("Asked:   %s"), strftime('%x %X', $request['time'])) . "\n";
            $text .= sprintf(_("Status:  %s"), formatStatus($request['status'])) . "\n";
            $text .= sprintf(_("URL:     %s"), Horde::applicationUrl('emailView.php?id=' . $request['id'], true, -1)) . "\n";
            $text .= _("First few lines:") . "\n" . String::wrap($request['text'], 76) . "\n";

            $i++;
            if ($i != 0 && $i != count($claimed)) {
                $text .= "\n--------------\n\n";
            }
        }
    }

    $text .= "\n\n---------------------------------------------------------------------------\n\n";
    $text .= String::wrap(sprintf(_("This email was automatically generated at %s by %s (%s)."),
                                strftime('%x %X'), $registry->get('name'), Horde::applicationUrl('', true, -1)));

    // Generate and send the email.
    $mime = &new MIME_Message();
    $mime->addPart(new MIME_Part('text/plain', $text));

    // Build the report headers.
    @list($user, $host) = @explode('@', $conf['stats']['warn_email_admin']);
    $from = MIME::rfc822WriteAddress($user, $host, sprintf(_("%s Administrator"), $registry->get('name')));
    $msg_headers = &new MIME_Headers();
    $msg_headers->addMessageIdHeader();
    $msg_headers->addHeader('Date', date('r'));
    $msg_headers->addHeader('From', $from);
    $msg_headers->addHeader('To', $conf['stats']['warn_email_admin']);
    $msg_headers->addHeader('Subject', sprintf(_("%s Stale Email Report"), $registry->get('name')));
    $msg_headers->addMIMEHeaders($mime);

    // Send the report.
    $result = $mime->send($conf['stats']['warn_email_admin'], $msg_headers);
    if (is_a($result, 'PEAR_Error')) {
        Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
    }
}

function formatStatus($status)
{
    if ($status == 'open') {
        return _("Open");
    } elseif ($status == 'claimed') {
        return _("Claimed");
    } elseif ($status == 'replied') {
        return _("Replied");
    } elseif ($status == 'forwarded') {
        return _("Forwarded");
    } elseif ($status == 'closed') {
        return _("Closed");
    }
}
