-- $Horde: rakim/scripts/sql/rakim.sql,v 1.2 2004/09/21 03:34:03 chuck Exp $

CREATE TABLE rakim_support_personnel (
    user_uid          VARCHAR(255) NOT NULL,
    user_last_active  INT,
--
    PRIMARY KEY (user_uid)
);

CREATE TABLE rakim_guests (
    guest_id                INT NOT NULL,
    session_id              VARCHAR(32),
    guest_name              VARCHAR(255) NOT NULL DEFAULT '',
    guest_email             VARCHAR(255) DEFAULT NULL,
    guest_organization      INT NOT NULL DEFAULT 1,
    guest_question_category VARCHAR(100),
    guest_type              VARCHAR(100),
    guest_remote_addr       VARCHAR(255) DEFAULT NULL,
    guest_user_agent        VARCHAR(255),
    guest_login_time        BIGINT NOT NULL,
    guest_logoff_time       BIGINT DEFAULT NULL,
    guest_referral_time     BIGINT DEFAULT NULL,
    guest_refresh_time      BIGINT DEFAULT NULL,
    guest_queue             INT DEFAULT NULL,
    guest_initial_question  VARCHAR(255),
    guest_entry_page        VARCHAR(255),
--
    PRIMARY KEY (guest_id)
);

CREATE TABLE rakim_queues (
    queue_id             INT NOT NULL DEFAULT 0,
    guest_id             INT NOT NULL DEFAULT 0,
    user_uid             VARCHAR(255),
    referrer_user_uid    VARCHAR(255),
    queue_status         VARCHAR(20) DEFAULT 'waiting',
--
    queue_create_time    BIGINT NOT NULL,
    queue_active_time    BIGINT,
    queue_exit_time      BIGINT,
    queue_question_types VARCHAR(255),
    queue_resolution     VARCHAR(100),
--
    PRIMARY KEY (queue_id)
);

CREATE TABLE rakim_organizations (
    organization_id    BIGINT NOT NULL,
    organization_name  VARCHAR(255) NOT NULL,
--
    PRIMARY KEY (organization_id)
);

CREATE TABLE rakim_messages (
    message_id      INT NOT NULL,
    message_text    TEXT,
    user_uid        VARCHAR(255) DEFAULT NULL,
    guest_id        INT DEFAULT NULL,
    message_sender  VARCHAR(100),
    message_time    INT DEFAULT NULL,
    message_push    CHAR(1) NOT NULL DEFAULT 'f',
    message_bounce  CHAR(1) NOT NULL DEFAULT 'f',
--
    PRIMARY KEY (message_id)
);

CREATE TABLE rakim_personnel_messages (
    pmessage_id        INT NOT NULL,
    pmessage_text      TEXT NOT NULL,
    user_uid_sender    VARCHAR(255) NOT NULL,
    user_uid_recipient VARCHAR(255) NOT NULL,
    pmessage_time      INT NOT NULL,
    pmessage_push      CHAR(1) NOT NULL DEFAULT 'f',
--
    PRIMARY KEY (pmessage_id)
);

CREATE INDEX rakim_support_messages_user_uid_sender_idx ON rakim_personnel_messages (user_uid_sender);
CREATE INDEX rakim_support_messages_user_uid_recipient_idx ON rakim_personnel_messages (user_uid_recipient);

CREATE TABLE rakim_canned_messages (
    canned_message_id     INT NOT NULL,
    user_uid              VARCHAR(255) NOT NULL,
    canned_message_title  VARCHAR(255) NOT NULL,
    canned_message_text   TEXT NOT NULL,
--
    PRIMARY KEY (canned_message_id)
);

CREATE INDEX rakim_canned_message_user_uid_idx ON rakim_canned_messages (user_uid);

CREATE TABLE rakim_emails (
    email_id                INT NOT NULL,
    email_text              TEXT,
    email_status            VARCHAR(10) NOT NULL DEFAULT 'open',
    email_user_uid          VARCHAR(255) NOT NULL,
--
    email_time_asked        INT NOT NULL,
    email_time_claimed      INT,
    email_time_answered     INT,
    email_time_closed       INT,
--
    guest_id                INT,
    guest_name              VARCHAR(255) NOT NULL DEFAULT '',
    guest_email             VARCHAR(255) NOT NULL DEFAULT '',
    guest_organization      INT NOT NULL DEFAULT 1,
    guest_question_category VARCHAR(100),
    guest_type              VARCHAR(100),
    guest_remote_addr       VARCHAR(255) DEFAULT NULL,
    guest_session_id        VARCHAR(32),
    guest_user_agent        VARCHAR(255),
--
    PRIMARY KEY (email_id)
);

CREATE TABLE rakim_email_responses (
    response_id       INT NOT NULL,
    email_id          INT NOT NULL,
    user_uid          VARCHAR(255) NOT NULL,
    response_time     INT NOT NULL,
    response_text     TEXT,
--
    PRIMARY KEY (response_id)
);

CREATE TABLE rakim_archived_sessions (
    session_id                INT NOT NULL,
    session_guest_id          INT NOT NULL,
    session_guest_type        VARCHAR(100),
    session_guest_login_time  BIGINT NOT NULL,
    session_guest_logoff_time BIGINT,
    session_guest_user_agent  VARCHAR(255),
    session_question_category VARCHAR(100),
    session_question_types    VARCHAR(255),
    session_resolution        VARCHAR(100),
    session_user_uid          VARCHAR(255),
    session_user_name         VARCHAR(255) NOT NULL,
    session_organization      VARCHAR(255) NOT NULL,
--
    session_create_time       BIGINT NOT NULL,
    session_active_time       BIGINT NOT NULL,
    session_exit_time         BIGINT NOT NULL,
--
    session_year              INT NOT NULL,
    session_month             SMALLINT NOT NULL,
    session_week              SMALLINT NOT NULL,
    session_day               SMALLINT NOT NULL,
    session_dow               SMALLINT NOT NULL,
    session_doy               SMALLINT NOT NULL,
    session_hour              SMALLINT NOT NULL,
--
    PRIMARY KEY (session_id)
);

CREATE TABLE rakim_archived_transcripts (
    transcript_id          INT NOT NULL,
    guest_id               INT NOT NULL,
    transcript_sender      VARCHAR(10) NOT NULL,
    transcript_time        INT NOT NULL,
    transcript_text        TEXT,
--
    PRIMARY KEY (transcript_id)
);

CREATE TABLE rakim_archived_emails (
    email_id                INT NOT NULL,
    email_text              TEXT,
--
    email_time_asked        INT NOT NULL,
    email_time_claimed      INT,
    email_time_answered     INT,
    email_time_closed       INT,
--
    email_year              INT NOT NULL,
    email_month             SMALLINT NOT NULL,
    email_week              SMALLINT NOT NULL,
    email_day               SMALLINT NOT NULL,
    email_dow               SMALLINT NOT NULL,
    email_doy               SMALLINT NOT NULL,
    email_hour              SMALLINT NOT NULL,
--
    guest_id                INT,
    guest_organization      VARCHAR(255) NOT NULL,
    guest_question_category VARCHAR(100),
    guest_type              VARCHAR(100),
    guest_user_agent        VARCHAR(255),
--
    PRIMARY KEY (email_id)
);

CREATE TABLE rakim_archived_email_responses (
    response_id       INT NOT NULL,
    email_id          INT NOT NULL,
    response_time     INT NOT NULL,
    response_text     TEXT,
--
    PRIMARY KEY (response_id)
);

GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_support_personnel TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_guests TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_queues TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_organizations TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_messages TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_personnel_messages TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_canned_messages TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_emails TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_email_responses TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_archived_sessions TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_archived_transcripts TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_archived_emails TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON rakim_archived_email_responses TO horde;
